
;;;;; (c) Remi van Trijp
;;;;;
;;;;; This file contains a sample grammar that can be tested for parsing and production.
;;;;; -----------------------------------------------------------------------------------------------------

(in-package :ldd-demo)

(def-constructions ldd-set
  :configuration ((:parse-goal-tests :no-applicable-cxns)
                  (:cxn-sets-with-sequential-application morph)
                  (:production-goal-tests :no-applicable-cxns)
                  (:create-initial-structure-mode . :root-node-mode)
                  (:de-render-mode . :de-render-with-sequence-and-precedes)
                  (:render-mode . :use-domains)
                  (:root-mode . t)
                  (:max-search-depth . 1000)
                  (:cxn-supplier-mode . :ordered-by-label)
                  (:production-order cxn morph)
                  (:parse-order morph cxn))

  ;; Speech Acts
  ;; -----------------------------------------------------------
  (def-cxn declarative-cxn (:label cxn) ;; The declarative construction is a "default" construction.
           ((root
             (tag ?meaning (meaning (== (assert ?ev)))))
            (?sv-anchor
             (sem-subunits (==p ?subject-unit ?vp-unit)))
            (?vp-unit
             (args (?ev))
             (sem-cat (==1 (sem-function predicating-expression))))
            ((J ?vp-unit)
             ?meaning))
           <-->
           ((?sv-anchor
             (syn-subunits (==p ?subject-unit ?vp-unit)))
            (?subject-unit
             (syn-cat (==1 (syn-role ?vp-unit subject)
                           (subcat detnp)))
             (form (==1 (boundaries ?subject-lb ?subject-rb))))
            (?vp-unit
             (footprints (==0 speech-act-cxn))
             (syn-cat (==1 (phrase-type vp)
                           (tam ?tam)
                           (agr (==1 (person ?p) (number ?n)))
                           (syn-valence (==1 (subject ?subject-unit)))
                           (finite-unit ?finite-unit)))
             (syn-subunits (== ?finite-unit))
             (form (==1 (boundaries ?finite-unit ?vp-rb))))
            (?finite-unit
             (syn-cat (==1 (verb-form finite)
                           (tam ?tam)
                           (agr (==1 (person ?p) (number ?n))))))
            ((J ?vp-unit)
             (footprints (== speech-act-cxn)))))

  (def-cxn interrogative-cxn (:label cxn)
           ((root
             (tag ?meaning (meaning (== (inquire-about ?ev ?participant)))))
            (?vp-unit
             (args (?ev))
             (sem-cat (==1 (sem-function predicating-expression))))
            (?np-unit
             (args (?participant . ?rest))
             (sem-cat (==1 (sem-function referring-expression)
                           (discourse-role topic))))
            ((J ?vp-unit)
             ?meaning))
           <-->
           ((?np-unit
             (footprints (== discourse-topic-cxn))
             (syn-cat (==1 (phrase-type np)
                           (subcat wh-np)))
             (form (==1 (boundaries ?np-lb ?np-rb))))
            (?vp-unit
             (footprints (==0 speech-act-cxn))
             (syn-cat (==1 (phrase-type vp)
                           (finite-unit ?finite-unit))))
            (root
             (form (== (meets ?np-rb ?finite-unit))))
            ((J ?vp-unit)
             (syn-subunits (== ?finite-unit))
             (footprints (== speech-act-cxn)))))
  
  (def-lex-entry the
                 :meaning (== (determined ?entity ?set))
                 :args (?entity ?set)
                 :sem-cat (==1 (class determiner))
                 :syn-cat (==1 (lex-id [the])
                               (lex-class article)
                               (agr ?agr)))

  (def-morph the :syn-cat (==1 (lex-id [the])
                               (lex-class article)
                               (agr ?agr)))

  (def-lex-entry brother
                 :meaning (== (brother ?set ?context))
                 :args (?set ?context)
                 :sem-cat (==1 (class entity)
                               (sex male))
                 :syn-cat (==1 (lex-id [brother])
                               (lex-class noun)
                               (agr (==1 (number sg)
                                         (person 3)))))
  
  (def-morph brother :syn-cat (==1 (lex-id [brother])
                                   (lex-class noun)
                                   (agr (==1 (number sg)
                                             (person 3)))))
  
  (def-lex-entry man
                 :meaning (== (man ?set ?context))
                 :args (?set ?context)
                 :sem-cat (==1 (class entity)
                               (sex male))
                 :syn-cat (==1 (lex-id [man])
                               (lex-class noun)
                               (agr (==1 (number sg)
                                         (person 3)))))
  
  (def-morph man :syn-cat (==1 (lex-id [man])
                                 (lex-class noun)
                                 (agr (==1 (number sg)
                                           (person 3)))))

  (def-lex-entry woman
                 :meaning (== (woman ?set ?context))
                 :args (?set ?context)
                 :sem-cat (==1 (class entity)
                               (sex female))
                 :syn-cat (==1 (lex-id [woman])
                               (lex-class noun)
                               (agr (==1 (number sg)
                                         (person 3)))))
  
  (def-morph woman :syn-cat (==1 (lex-id [woman])
                                 (lex-class noun)
                                 (agr (==1 (number sg)
                                           (person 3)))))

  (def-cxn discourse-topic (:label cxn)
           ((root
             (tag ?meaning (meaning (== (discourse-topic ?topic)))))
            (?first-unit
             (args (?topic . ?rest))
             (sem-cat (==1 (class ?any-class))))
            (?phrasal-unit
             (sem-cat (==1 (sem-function referring-expression)))
             (sem-subunits (== ?first-unit)))
            ((J ?phrasal-unit)
             (sem-cat (==1 (discourse-role topic))))
            ((J ?first-unit)
             ?meaning))
           <-->
           ((root
             (form (== (sequence ?first-unit . ?other-units))))
            (?phrasal-unit
             (syn-cat (==1 (phrase-type np)))
             (syn-subunits (== ?first-unit))
             (footprints (==0 discourse-topic-cxn)))
            ((J ?phrasal-unit)
             (syn-cat (==1 (discourse-role topic)))
             (footprints (== discourse-topic-cxn)))))

    (def-cxn who-cxn (:label cxn)
             ((root
               (tag ?meaning (meaning (== (inquire-about-human-referent ?ref ?context)))))
              ((J ?who-unit)
               ?meaning
               (args (?ref ?context))
               (sem-cat ((class inquiry))))
              ((J ?np-unit)
               ?meaning
               (sem-subunits (?who-unit))
               (args (?ref))
               (sem-cat ((sem-function referring-expression)
                         (sem-role ?sem-role)
                         (head-unit ?who-unit)))))
             <-->
             ((root
               (tag ?form (form (== (string ?who-unit "who")))))
              ((J ?who-unit)
               ?form
               (syn-cat ((lex-class interrogative-pronoun))))
              ((J ?np-unit)
               (syn-subunits (?who-unit))
               (form ((boundaries ?who-unit ?who-unit)))
               (syn-cat (==1 (phrase-type np)
                             (subcat wh-np)
                             (syn-role ?vp-unit ?syn-role)
                             (agr (==1 (number ?nr)
                                       (person 3))))))))         
  (def-cxn det-np-cxn (:label cxn)
           ((?article-unit
             (args (?entity ?set))
             ;; (sem-subunits (root))
             (sem-cat (==1 (class determiner))))
            (?noun-unit
             (args (?set ?context))
             (footprints (==0 np-cxn))
             ;; (sem-subunits (root))
             (sem-cat (==1 (class entity))))
            ((J ?np-unit)
             (args (?entity))
             (sem-subunits (?article-unit ?noun-unit))
             (sem-cat (==1 (sem-function referring-expression)
                           (head-unit ?noun-unit)
                           (sem-role ?sem-role))))
            ((J ?noun-unit)
             (footprints (== np-cxn))))
           <-->
           ((root
             (form (== (meets ?article-unit ?noun-unit))))
            (?article-unit
             (syn-cat (==1 (lex-class article)
                           (agr ?agr))))
            (?noun-unit
             (footprints (==0 np))
             (syn-cat (==1 (lex-class noun)
                           (agr ?agr))))
            ((J ?np-unit)
             (syn-subunits (?article-unit ?noun-unit))
             (form (==1 (boundaries ?article-unit ?noun-unit)))
             (syn-cat (==1 (phrase-type np)
                           (subcat detnp)
                           (syn-role ?vp-unit ?syn-role)
                           (agr ?agr))))
            ((J ?noun-unit)
             (footprints (== np)))))

  ;; Lexical verbs and their verb forms:
  ;; --------------------------------------------------------------------------------
  (def-lex-entry think
                 :meaning (== (think ?ev)
                              (thinker ?ev ?thinker)
                              (thought ?ev ?thought))
                 :args (?ev)
                 :sem-cat (==1 (class event)
                               (sem-valence (==1 (agent ?ev ?thinker)
                                                 (patient ?ev ?thought))))
                 :syn-cat (==1 (lex-id [think])
                               (lex-class verb)
                               (subclass cognizer-verb)
                               (verb-form ?verb-form)
                               (syn-valence (==1 (subject ?subject-unit)
                                                 (object ?complement-unit)))
                               (tam ?tam)
                               (agr (==1 (number ?nr) (person ?p)))))

  (def-morph think :syn-cat (==1 (lex-id [think])
                                 (lex-class verb)
                                 (subclass cognizer-verb)
                                 (verb-form ?verb-form)
                                 (syn-valence (==1 (subject ?subject-unit)
                                                   (object ?complement-unit)))
                                 (tam ?tam)
                                 (agr (==1 (number ?nr) (person ?p)))))

  (def-lex-entry see
                 :meaning (== (see ?ev)
                              (seer ?ev ?seer)
                              (seen ?ev ?seen))
                 :args (?ev)
                 :sem-cat (==1 (class event)
                               (sem-valence (==1 (agent ?ev ?seer)
                                                 (patient ?ev ?seen))))
                 :syn-cat (==1 (lex-id [see])
                               (lex-class verb)
                               (subclass perception-verb)
                               (verb-form ?vb)
                               (syn-valence (==1 (subject ?subject-unit)
                                                 (object ?object-unit)))
                               (agr (==1 (number ?number)
                                         (person ?person)))))

  (def-morph see :syn-cat (==1 (lex-id [see])
                               (lex-class verb)
                               (subclass perception-verb)
                               (verb-form ?vf)
                               (syn-valence (==1 (subject ?subject-unit)
                                                 (object ?object-unit)))
                               (TAM ((tense ?tense)))
                               ;; TODO: ASPECT LATER (aspect (non-progressive))))
                               (agr (==1 (number ?nr)
                                         (person ?person)))))

  (def-morph did :syn-cat (==1 (lex-id [do])
                               (lex-class aux)
                               (TAM ((tense past)))
                               (agr (==1 (number ?nr)
                                         (person ?person)))))

  (def-morph saw :syn-cat (==1 (lex-id [see])
                               (lex-class verb)
                               (verb-form finite)
                               (syn-valence (==1 (subject ?subject-unit)
                                                 (object ?object-unit)))
                               (TAM ((tense past)))
                               ;; TODO: ASPECT LATER (aspect (non-progressive))))
                               (agr (==1 (number sg)
                                         (person 3)))))

  (def-cxn verb-phrase-cxn (:label cxn)
           ((?main-verb-unit
             (footprints (==0 verb-phrase-cxn))
             ;; (sem-subunits (root))
             (args (?ev))
             (sem-cat (==1 (class event)
                           (sem-valence ?sem-valence))))
            ((J ?vp-unit)
             (sem-subunits (?main-verb-unit))
             (sem-cat (==1 (sem-function predicating-expression)
                           (sem-valence ?sem-valence)))
             (args (?ev)))
            ((J ?main-verb-unit)
             (footprints (== verb-phrase-cxn))))
           <-->
           ((?main-verb-unit
             (footprints (==0 verb-phrase-cxn))
             ;; (syn-subunits (root))
             (syn-cat (==1 (lex-class verb)
                           (tam ?tam)
                           (agr ?agr)
                           (syn-valence ?syn-valence))))
            ((J ?vp-unit)
             (syn-subunits (?main-verb-unit))
             (form (==1 (boundaries ?main-verb-unit ?main-verb-unit)))
             (syn-cat (==1 (syn-function verbal)
                           (finite-unit ?finite-unit)
                           (phrase-type vp)                           
                           (lex-head ?main-verb-unit)
                           (tam ?tam)
                           (agr ?agr)
                           (syn-valence ?syn-valence))))
            ((J ?main-verb-unit)
             (footprints (== verb-phrase-cxn)))))
  
  ;; Transitive Construction.
  ;; -----------------------------------------------------------
  (def-cxn active-transitive-cxn (:label cxn)
           ((root
             (tag ?meaning (meaning (== (frame ?event [ACT])
                                        (actor ?event ?agent)
                                        (undergoer ?event ?patient)))))
            (?sv-anchor-unit
             (sem-subunits (==p ?agent-unit ?event-unit))
             (sem-cat (==1 (sem-function deictic))))
            (?event-unit
             (args (?event))
             (sem-cat (==1 (sem-function predicating-expression)
                           (sem-valence (==1 (agent ?event ?agent)
                                             (patient ?event ?patient))))))
            (?agent-unit
             (args (?agent))
             (sem-cat (==1 (sem-function referring-expression)
                           (sem-role agent))))
            (?patient-unit
             (args (?patient))
             (sem-cat (==1 (sem-function referring-expression)
                           (sem-role patient))))
            ((J ?transitive-clause-unit)
             ?meaning
             (sem-subunits (?event-unit ?agent-unit ?patient-unit))
             (args (?event ?agent ?patient))
             (sem-cat ((sem-function proposition)))))
           <-->
           ((?sv-anchor-unit
             (syn-subunits (==p ?agent-unit ?event-unit))
             (syn-cat (==1 (syn-function anchorring))))
            (?event-unit
             (syn-cat (==1 (phrase-type vp)
                           (syn-valence (==1 (subject ?agent-unit)
                                             (object ?patient-unit)))
                           (agr (==1 (number ?nr) (person ?person)))))
             (footprints (==0 arg-cxn)))
            (?agent-unit
             (syn-cat (==1 (syn-role ?event-unit subject)
                           (agr (==1 (number ?nr) (person ?person))))))
            (?patient-unit
             (syn-cat (==1 (syn-role ?event-unit object))))
            ((J ?transitive-clause-unit)
             (syn-subunits (?event-unit ?agent-unit ?patient-unit))
             (syn-cat ((clause-type transitive-clause)
                       (voice active))))
            ((J ?event-unit)
             (footprints (== arg-cxn)))))

  ;; SV-anchors.
  ;; -----------------------------------------------------------                                        
  (def-cxn sv-anchor-cxn (:label cxn)
           ((root
             (tag ?meaning (meaning (== (deictic-perspective ?ev ?participant)))))
            (?vp-unit
             (args (?ev))
             (sem-subunits (?lb-vp))
             (sem-cat (==1 (sem-function predicating-expression))))
            (?subject-unit
             (args (?participant))
             (sem-cat (==1 (sem-function referring-expression)
                           (head-unit ?rb-subj))))
            ((J ?sv-anchor-unit)
             ?meaning
             (sem-subunits (?subject-unit ?vp-unit))
             (sem-cat ((sem-function deictic)))))
           <-->
           ((?vp-unit
             (form (==1 (boundaries ?lb-vp ?rb-vp)))
             (syn-cat (==1 (phrase-type vp)
                           (syn-valence (==1 (subject ?subject-unit)))
                           (agr (==1 (number ?nr) (person ?person)))))
             (footprints (==0 sv-anchor-cxn)))
            (?subject-unit
             (form (==1 (boundaries ?lb-subj ?rb-subj)))
             (syn-cat (==1 (phrase-type np)
                           (syn-role ?vp-unit subject)
                           (agr (==1 (number ?nr) (person ?person))))))
            (root
             (form (== (meets ?rb-subj ?lb-vp))))
            ((J ?sv-anchor-unit)
             (syn-subunits (?subject-unit ?vp-unit))
             (form (== (boundaries ?lb-subj ?rb-vp)))
             (syn-cat (==1 (syn-function anchorring))))
            ((J ?vp-unit)
             (footprints (== sv-anchor-cxn)))))

  ;; DO-SUPPORT
  ;; -----------------------------------------------------------
  (def-syn-cxn do-sv-anchor-cxn (:label cxn)
               ((root)
                (?vp-unit
                 (footprints (==0 do-sv-anchor-cxn))
                 (syn-cat (==1! (phrase-type vp)
                                (finite-unit ?aux-do-unit)
                                (tam ?tam)
                                (agr ?agr)))
                 (syn-subunits (==p ?aux-do-unit ?main-verb-unit)))
                (?aux-do-unit
                 (syn-cat (==1! (lex-id [do])
                                (lex-class aux)
                                (tam ?tam)
                                (agr ?agr))))
                (?main-verb-unit
                 (syn-cat (==1! (verb-form non-finite))))
                ((J ?vp-unit)
                 (footprints (== do-sv-anchor-cxn))))
               <-->
               ((?sv-anchor-unit
                 (syn-cat (==1 (syn-function anchorring)))
                 (form (==1 (boundaries ?subj-lb ?main-verb-unit))))
                (?aux-do-unit
                 (footprints (==0 do-sv-anchor-cxn))
                 (syn-cat (==1 (lex-id [do]))))
                (root
                 (form (== (meets ?aux-do-unit ?subj-lb))))
                ((J ?aux-do-unit)
                 (footprints (== do-sv-anchor-cxn)))))
  
  (def-syn-cxn do-support-cxn (:label cxn)
               ((?topical-phrase-unit
                 (footprints (==0 do-support-cxn))
                 (syn-cat (==1 (syn-role ?vp-unit object)
                               (phrase-type np)
                               (subcat wh-np)
                               (discourse-role topic))))
                ((J ?topical-phrase-unit)
                 (footprints (== do-support-cxn)))
                ((J ?aux-do-unit)))
               <-->
               ((?topical-phrase-unit
                 (form (== (boundaries ?lb-topic ?rb-topic))))
                (?aux-do-unit
                 (footprints (==0 do-support-cxn))
                 (syn-cat (==1 (lex-id [do])
                               (lex-class aux)
                               (tam ?tam)
                               (agr ?agr))))
                (root
                 (form (== (meets ?rb-topic ?aux-do-unit))))
                ((J ?aux-do-unit)
                 (footprints (== do-support-cxn))))))