
;;;;; (c) Remi van Trijp

(in-package :ldd-demo)

(activate-monitor trace-fcg)

(setf *create-static-expandable/collapsable-elements* t)

(defun make-introduction-page ()
  (clear-page)
  (create-static-html-page "Long-Distance Dependencies in FCG"
    (header "Web demonstration:" 'h3)
    (header "Long-Distance Dependencies in Fluid Construction Grammar")
    (browser-warning)
    (paragraph "This demonstration shows how long-distance dependencies can be processed handled in FCG. It supports the following article:")
    (bullet "van Trijp, Remi (under review). Long-Distance Dependencies without Filler-Gaps. A Cognitive-Functional Alternative in Fluid Construction Grammar.")
    (paragraph "If you are new to Fluid Construction Grammar, please check our <a href=\"http://www.fcg-net.org/projects/web-demonstration-guide/\"> web demonstration guide</a> to learn more about what you can see in the demo and what you can do with it. The web demonstration also assumes the visitor to be familiar with how FCG handles lexical, phrasal and argument structure constructions, as well as the concept of linguistic search. Background reading and demonstrations are available at this demo page.")
    (hline)
    (header "Contents of this Demonstration" 'h3)
    (bullet "<a href=\"sample-grammar/index.html\">The sample grammar</a>")
    (bullet "<a href=\"topicalization/index.html\">Topicalization</a>")
    (bullet "<a href=\"questions/index.html\">WH-questions</a>")
    (hline)
    (paragraph "(c) <a href=\"http://www.remivantrijp.be\">Remi van Trijp</a>")))

(defun show-sample-grammar ()
  (activate-monitor trace-fcg)
  (make-ldd-set-cxns)
  (clear-page)
  (create-static-html-page "A sample grammar"
    (header "A sample grammar" 'h3)
    (paragraph "You can download a sample grammar by <a href=\"ldd.zip\">clicking on this link.</a>")
    (paragraph "The grammar allows you to parse and produce some elementary topicalization patterns and questions in English. It also contains template definitions for the following constructions that are used in the demonstrations:")
    (show-inventory :expand-initially t)))

(defun demo-topicalization ()
  (activate-monitor trace-fcg)
  (clear-page)
  (create-static-html-page "Topicalization"
    (header "Processing Topicalization Patterns" 'h2)
    (paragraph "Topicalization has traditionally been analyzed in terms of a special topicalization construction, or through movement. This demonstration shows that such an analysis is not necessary: the same constructions that are able to process utterances with canonical word order, are also able to handle so-called topicalization. The only difference is that the subject and the topic of the utterance do not coincide.")
    (parse '("the" "woman" "saw" "the" "man") *constructions*)
    (parse '("the" "man" "the" "woman" "saw") *constructions*)
    (produce '((assert ev-1)
               (discourse-topic the-woman)
               (deictic-perspective ev-1 the-woman)
               (determined the-woman women) (woman women context)
               (see ev-1) (seer ev-1 the-woman) (seen ev-1 the-man)
               (determined the-man men) (man men context)
               (frame ev-1 [act]) (actor ev-1 the-woman) (undergoer ev-1 the-man)) *constructions*)
    (produce '((assert ev-1)
               (discourse-topic the-man)
               (deictic-perspective ev-1 the-woman)
               (determined the-woman women) (woman women context)
               (see ev-1) (seer ev-1 the-woman) (seen ev-1 the-man)
               (determined the-man men) (man men context)
               (frame ev-1 [act]) (actor ev-1 the-woman) (undergoer ev-1 the-man)) *constructions*)))

(defun demo-wh-questions ()
  (activate-monitor trace-fcg)
  (clear-page)
  (create-static-html-page "WH-Questions"
    (header "Processing WH-questions" 'h2)
    (paragraph "Most linguistic analyses treat different kinds of interrogatives as different kinds of constructions. Here again, this demonstration shows that different patterns in interrogatives can be explained as a side-effect of language processing. More specifically, WH-interrogatives require finite verb to follow the topic/focus of the
question. DO-support is called only when no auxiliary can be recruited.")
    (parse '("who" "saw" "the" "woman") *constructions*)
    (parse '("who" "did" "the" "man" "see") *constructions*)
    (produce '((inquire-about ev-1 a-person)
               (discourse-topic a-person)
               (deictic-perspective ev-1 a-person)
               (inquire-about-human-referent a-person context)
               (see ev-1) (seer ev-1 a-person) (seen ev-1 the-man)
               (determined the-man men) (man men context)
               (frame ev-1 [act]) (actor ev-1 a-person) (undergoer ev-1 the-man)) *constructions*)
    (produce '((inquire-about ev-1 a-person)
               (discourse-topic a-person)
               (deictic-perspective ev-1 the-man)
               (inquire-about-human-referent a-person context)
               (see ev-1) (seer ev-1 the-man) (seen ev-1 a-person)
               (determined the-man men) (man men context)
               (frame ev-1 [act]) (actor ev-1 the-man) (undergoer ev-1 a-person)) *constructions*)))

    
;; (clear-page)
;; (make-introduction-page)
;; (show-sample-grammar)
;; (demo-topicalization)
