
;;;;; (c) Remi van Trijp
;;;;;
;;;;; This file contains templates that are used by the sample grammar for defining constructions.
;;;;; -----------------------------------------------------------------------------------------------------

(in-package :ldd-demo)

;;;;; #########################################################################################
;;;;; DEF-SYN-CXN: For defining construction that only operate on the syntactic pole.
;;;;; #########################################################################################

(defmacro def-syn-cxn (name attributes left-pole arrow right-pole)
  (declare (ignore arrow))
  `(add-cxn (make-instance 'construction
                           :name ',name
                           :domain 'syn
                           :attributes `,(fcg::parse-attributes ',attributes)
                           :left-pole ',left-pole
                           :right-pole ',right-pole)
            *constructions*))

;;;;; #########################################################################################
;;;;; WORD FORMS: INTERFACE WITH SPEECH/GESTURE/WRITING
;;;;; #########################################################################################

(defmacro def-morph (name &key syn-cat)
  (let ((?lex-unit (make-var (format nil "~a-unit" name))))
  `(def-syn-cxn ,(make-symbol (format nil "~a-morph" name)) (:label morph)
                ((,?lex-unit
                  (syn-cat ,syn-cat)
                  (footprints (==0 morph)))
                 ((J ,?lex-unit)
                  (footprints (== morph))))
                <-->
                ((root
                  (tag ?form (form (== (string ,?lex-unit ,(string-downcase (symbol-name name)))))))
                 ((J ,?lex-unit)
                  ?form)))))

(defmacro def-word-form (name &key fcg-class)
  `(def-syn-cxn ,(make-symbol (format nil "~a-morph" name)) (:label morph)
                ((?lex-unit
                  (syn-subunits (root))
                  (syn-cat (++ :fcg-class ,(if (listp fcg-class)
                                             fcg-class
                                             (list fcg-class))))
                  (footprints (==0 morph)))
                 ((J ?lex-unit)
                  (footprints (== morph))))
                <-->
                ((root
                  (tag ?form (form (== (string ?lex-unit ,(if (numberp name)
                                                            (write-to-string name)
                                                            (string-downcase (symbol-name name))))))))
                 ((J ?lex-unit)
                  (syn-subunits (root))
                  ?form))))

;;;;; #########################################################################################
;;;;; LEXICAL ENTRIES
;;;;; #########################################################################################

(defun add-lex-claim (syn-cat-spec)
  "Add the feature-value pair (claimed-by lex) to the spec."
  (if (eql '++ (first syn-cat-spec))
    (let* ((class (first (third syn-cat-spec)))
           (class-spec (second (third syn-cat-spec))))
      `(++ :fcg-class ,(list class
                             (cons '(claimed-by lex) class-spec))))
    syn-cat-spec))

(defmacro def-lex-entry (name &key meaning args sem-cat syn-cat)
  (let ((?word-unit (make-var (format nil "~a" name))))
    `(def-cxn ,(make-cxn-name name 'lex) (:label cxn)
              ((root
                (tag ?meaning (meaning ,meaning)))
               ((J ,?word-unit)
                ?meaning
                (footprints (== lex))
                (args ,args)
                (sem-cat ,sem-cat)))
              <-->
              ((,?word-unit
                (syn-cat ,(add-lex-claim syn-cat))
                (footprints (==0 lex)))
               ((J ,?word-unit)
                (footprints (== lex)))))))
